#include <cmath>

#include "vecteur.h"

namespace m2if {

// structors

Vect::Vect(int size)
{
  if (size <= 0)
    throw "wrong size";

  size_ = size;

  data_ = new double[size_];
  if (!data_)
    throw "memory allocation failed";
}

Vect::Vect(const Vect & v)
{
  size_ = v.size_;
  data_ = new double[size_];
  if (!data_)
    throw "memory allocation failed";

  for (int i = 0; i < size_; i++)
    data_[i] = v.data_[i];
}

Vect::~Vect()
{
  delete [] data_;
}

// getters

double Vect::operator[](int i) const
{
  if ((i < 0) || (i >= size_))
    throw "wrong index";

  return data_[i];
}

// setters

double & Vect::operator[](int i)
{
  if ((i < 0) || (i >= size_))
    throw "wrong index";

  return data_[i];
}

// assignment

Vect & Vect::operator=(const Vect & v)
{
  if (this != &v)
    {
      Vect tmp = v;
      double *d;

      size_ = tmp.size_;
      d = tmp.data_;
      tmp.data_ = data_;
      data_ = d;
    }

  return *this;
}

// math functions

double Vect::norm()
{
  return sqrt(*this * *this);
}

std::ostream & operator<<(std::ostream & st, const Vect & v)
{
  st << "(";
  for (int i = 0; i < v.size_get(); i++)
    {
      st << v[i];
      if (i != (v.size_get() - 1))
	st << ",";
    }
  st << ")";

  return st;
}

Vect operator+(const Vect & v1, const Vect & v2)
{
  if (v1.size_get() != v2.size_get())
    throw "wrong size";

  Vect v(v1.size_get());

  for (int i = 0; i < v1.size_get(); i++)
    v[i] = v1[i] + v2[i];

  return v;
}

Vect operator-(const Vect & v1, const Vect & v2)
{
  if (v1.size_get() != v2.size_get())
    throw "wrong size";

  Vect v(v1.size_get());

  for (int i = 0; i < v1.size_get(); i++)
    v[i] = v1[i] + v2[i];

  return v;
}

double operator*(const Vect & v1, const Vect & v2)
{
  if (v1.size_get() != v2.size_get())
    throw "wrong size";

  double sum = 0.0;
  for (int i = 0; i < v1.size_get(); i++)
    {
      sum += v1[i] * v2[i];
    }

  return sum;
}



} // namespace m2if
